@@ -1,21 +1,17 @@
+
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  common.cpp
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: common.cpp 2802 2005-02-20 18:40:37Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   helper functions, common variables
 
-/*!
-    \file
-    \version $Id: common.cpp 1494 2004-02-12 18:41:07Z mosu $
-    \brief helper functions, common variables
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include "os.h"
@@ -31,25 +27,37 @@
 #endif
 #include <locale.h>
 #include <stdarg.h>
-#include <stdio.h>
+#if defined(COMP_GCC) && (__GNUC__ < 3)
+# define __USE_ISOC99
+# include <stdio.h>
+# undef __USE_ISOC99
+#else
+# include <stdio.h>
+#endif // COMP_GCC && __GNUC__ < 3
 #include <stdlib.h>
 #include <string.h>
+#if !defined(COMP_MSC)
 #include <sys/time.h>
+#endif
 #include <time.h>
 #include <wchar.h>
 #if defined(SYS_WINDOWS)
-#include <windef.h>
-#include <winbase.h>
-#include <winnls.h>
+#include <windows.h>
 #endif
 
 #include <string>
 #include <vector>
 
-using namespace std;
+#include "ebml/EbmlBinary.h"
 
+using namespace std;
+using namespace libebml;
+
+#include "base64.h"
+#include "common.h"
+#include "error.h"
 #include "hacks.h"
-#include "mkvmerge.h"
+#include "mm_io.h"
 
 int verbose = 1;
 bool suppress_warnings = false;
@@ -67,58 +75,74 @@ bitvalue_c::bitvalue_c(const bitvalue_c &src) {
   *this = src;
 }
 
-#define ishexdigit(c) (isdigit(c) || (((c) >= 'a') && ((c) <= 'f')) || \
-                       (((c) >= 'A') && ((c) <= 'F')))
-#define upperchar(c) (((c) >= 'a') ? ((c) - 'a' + 'A') : (c))
-#define hextodec(c) (isdigit(c) ? ((c) - '0') : ((c) - 'A' + 10))
+#define ishexdigit(c) (isdigit(c) || (((c) >= 'a') && ((c) <= 'f')))
+#define hextodec(c) (isdigit(c) ? ((c) - '0') : ((c) - 'a' + 10))
 
-bitvalue_c::bitvalue_c(const char *s, int allowed_bitlength) {
+bitvalue_c::bitvalue_c(string s,
+                       int allowed_bitlength) {
   int len, i;
+  bool previous_was_space;
   string s2;
 
-  len = strlen(s);
-  if (len < 2)
-    throw exception();
+  if ((allowed_bitlength != -1) && ((allowed_bitlength % 8) != 0))
+    throw error_c("wrong usage: invalid allowed_bitlength");
 
-  if ((s[0] == '0') && (s[1] == 'x')) {
-    i = 0;
-    while (i < len) {
-      if ((len - i) < 4)
-        throw exception();
-      if ((s[i] != '0') || (s[i + 1] != 'x'))
-        throw exception();
-      s2 += s[i + 2];
-      s2 += s[i + 3];
-      i += 4;
-      if (i < len) {
-        if (s[i] != ' ')
-          throw exception();
-        i++;
-      }
+  len = s.size();
+  previous_was_space = true;
+  s = downcase(s);
+
+  for (i = 0; i < len; i++) {
+    // Space or tab?
+    if (isblanktab(s[i])) {
+      previous_was_space = true;
+      continue;
     }
-    s = s2.c_str();
-    len = strlen(s);
+    previous_was_space = false;
+
+    // Skip hyphens and curly braces. Makes copy & paste a bit easier.
+    if ((s[i] == '-') || (s[i] == '{') || (s[i] == '}'))
+      continue;
+
+    // Space or tab followed by "0x"? Then skip it.
+    if (s.substr(i, 2) == "0x") {
+      i++;
+      continue;
+    }
+
+    // Invalid character?
+    if (!ishexdigit(s[i]))
+      throw error_c(mxsprintf("not a hex digit at position %d", i));
+
+    // Input too long?
+    if ((allowed_bitlength > 0) && ((s2.length() * 4) >= allowed_bitlength))
+      throw error_c(mxsprintf("input too long: %d > %d", s2.length() * 4,
+                              allowed_bitlength));
+
+    // Store the value.
+    s2 += s[i];
   }
 
-  if ((len % 2) == 1)
-    throw exception();
-
-  if ((allowed_bitlength != -1) && ((len * 4) != allowed_bitlength))
-    throw exception();
-
-  for (i = 0; i < len; i++)
-    if (!ishexdigit(s[i]))
-      throw exception();
+  // Is half a byte or more missing?
+  len = s2.length();
+  if (((len % 2) != 0)
+      ||
+      ((allowed_bitlength != -1) && ((len * 4) < allowed_bitlength)))
+    throw error_c("missing one hex digit");
 
   value = (unsigned char *)safemalloc(len / 2);
   bitsize = len * 4;
 
   for (i = 0; i < len; i += 2)
-    value[i / 2] = hextodec(upperchar(s[i])) * 16 +
-      hextodec(upperchar(s[i + 1]));
+    value[i / 2] = hextodec(s2[i]) << 4 | hextodec(s2[i + 1]);
 }
 
-bitvalue_c &bitvalue_c::operator =(const bitvalue_c &src) {
+bitvalue_c::bitvalue_c(const EbmlBinary &elt) {
+  bitsize = elt.GetSize() << 3;
+  value = (unsigned char *)safememdup(elt.GetBuffer(), elt.GetSize());
+}
+
+bitvalue_c &
+bitvalue_c::operator =(const bitvalue_c &src) {
   safefree(value);
   bitsize = src.bitsize;
   value = (unsigned char *)safememdup(src.value, bitsize / 8);
@@ -130,132 +154,48 @@ bitvalue_c::~bitvalue_c() {
   safefree(value);
 }
 
-bool bitvalue_c::operator ==(const bitvalue_c &cmp) const {
-  int i;
-
+bool
+bitvalue_c::operator ==(const bitvalue_c &cmp)
+  const {
   if (cmp.bitsize != bitsize)
     return false;
-  for (i = 0; i < bitsize / 8; i++)
-    if (value[i] != cmp.value[i])
-      return false;
-  return true;
+  return memcmp(value, cmp.value, bitsize / 8) == 0;
 }
 
-unsigned char bitvalue_c::operator [](int index) const {
+unsigned char
+bitvalue_c::operator [](int index)
+  const {
   assert((index >= 0) && (index < (bitsize / 8)));
   return value[index];
 }
 
-int bitvalue_c::size() const {
+int
+bitvalue_c::size()
+  const {
   return bitsize;
 }
 
-unsigned char *bitvalue_c::data() const {
+unsigned char *
+bitvalue_c::data()
+  const {
   return value;
 }
 
-void bitvalue_c::generate_random() {
+void
+bitvalue_c::generate_random() {
   int i;
 
   for (i = 0; i < bitsize / 8; i++)
     value[i] = (unsigned char)(255.0 * rand() / RAND_MAX);
 }
 
-// ---------------------------------------------------------------------
-
-byte_cursor_c::byte_cursor_c(const unsigned char *ndata, int nsize) :
-  size(nsize),
-  data(ndata) {
-  pos = 0;
-  if (nsize < 0)
-    throw exception();
-}
-
-unsigned char byte_cursor_c::get_uint8() {
-  if ((pos + 1) > size)
-    throw exception();
-
-  pos++;
-
-  return data[pos - 1];
-}
-
-unsigned short byte_cursor_c::get_uint16_be() {
-  unsigned short v;
-
-  if ((pos + 2) > size)
-    throw exception();
-
-  v = ::get_uint16_be(&data[pos]);
-  pos += 2;
-
-  return v;
-}
-
-unsigned int byte_cursor_c::get_uint32_be() {
-  unsigned int v;
-
-  if ((pos + 4) > size)
-    throw exception();
-
-  v = ::get_uint32_be(&data[pos]);
-  pos += 4;
-
-  return v;
-}
-
-unsigned short byte_cursor_c::get_uint16() {
-  unsigned short v;
-
-  if ((pos + 2) > size)
-    throw exception();
-
-  v = ::get_uint16(&data[pos]);
-  pos += 2;
-
-  return v;
-}
-
-unsigned int byte_cursor_c::get_uint32() {
-  unsigned int v;
-
-  if ((pos + 4) > size)
-    throw exception();
-
-  v = ::get_uint32(&data[pos]);
-  pos += 4;
-
-  return v;
-}
-
-void byte_cursor_c::skip(int n) {
-  if ((pos + n) > size)
-    throw exception();
-
-  pos += n;
-}
-
-void byte_cursor_c::get_bytes(unsigned char *dst, int n) {
-  if ((pos + n) > size)
-    throw exception();
-
-  memcpy(dst, &data[pos], n);
-  pos += n;
-}
-
-int byte_cursor_c::get_pos() {
-  return pos;
-}
-
-int byte_cursor_c::get_len() {
-  return size - pos;
-}
-
 /*
- * Control functions
- */
+   Control functions
+*/
 
-void die(const char *fmt, ...) {
+void
+die(const char *fmt,
+    ...) {
   va_list ap;
 
   mxprint(stdout, "'die' called: ");
@@ -269,15 +209,19 @@ void die(const char *fmt, ...) {
   exit(2);
 }
 
-void _trace(const char *func, const char *file, int line) {
+void
+_trace(const char *func,
+       const char *file,
+       int line) {
   mxdebug("trace: %s:%s (%d)\n", file, func, line);
 }
 
 /*
- * Endianess stuff
- */
+   Endianess stuff
+*/
 
-uint16_t get_uint16(const void *buf) {
+uint16_t
+get_uint16_le(const void *buf) {
   uint16_t ret;
   unsigned char *tmp;
 
@@ -289,7 +233,8 @@ uint16_t get_uint16(const void *buf) {
   return ret;
 }
 
-uint32_t get_uint24(const void *buf) {
+uint32_t
+get_uint24_le(const void *buf) {
   uint32_t ret;
   unsigned char *tmp;
 
@@ -302,7 +247,8 @@ uint32_t get_uint24(const void *buf) {
   return ret;
 }
 
-uint32_t get_uint32(const void *buf) {
+uint32_t
+get_uint32_le(const void *buf) {
   uint32_t ret;
   unsigned char *tmp;
 
@@ -316,7 +262,8 @@ uint32_t get_uint32(const void *buf) {
   return ret;
 }
 
-uint64_t get_uint64(const void *buf) {
+uint64_t
+get_uint64_le(const void *buf) {
   uint64_t ret;
   unsigned char *tmp;
 
@@ -334,7 +281,8 @@ uint64_t get_uint64(const void *buf) {
   return ret;
 }
 
-uint16_t get_uint16_be(const void *buf) {
+uint16_t
+get_uint16_be(const void *buf) {
   uint16_t ret;
   unsigned char *tmp;
 
@@ -346,7 +294,8 @@ uint16_t get_uint16_be(const void *buf) {
   return ret;
 }
 
-uint32_t get_uint24_be(const void *buf) {
+uint32_t
+get_uint24_be(const void *buf) {
   uint32_t ret;
   unsigned char *tmp;
 
@@ -359,7 +308,8 @@ uint32_t get_uint24_be(const void *buf) {
   return ret;
 }
 
-uint32_t get_uint32_be(const void *buf) {
+uint32_t
+get_uint32_be(const void *buf) {
   uint32_t ret;
   unsigned char *tmp;
 
@@ -373,7 +323,8 @@ uint32_t get_uint32_be(const void *buf) {
   return ret;
 }
 
-uint64_t get_uint64_be(const void *buf) {
+uint64_t
+get_uint64_be(const void *buf) {
   uint64_t ret;
   unsigned char *tmp;
 
@@ -391,7 +342,9 @@ uint64_t get_uint64_be(const void *buf) {
   return ret;
 }
 
-void put_uint16(void *buf, uint16_t value) {
+void
+put_uint16_le(void *buf,
+              uint16_t value) {
   unsigned char *tmp;
 
   tmp = (unsigned char *) buf;
@@ -400,7 +353,9 @@ void put_uint16(void *buf, uint16_t value) {
   tmp[1] = (value >>= 8) & 0xff;
 }
 
-void put_uint32(void *buf, uint32_t value) {
+void
+put_uint32_le(void *buf,
+              uint32_t value) {
   unsigned char *tmp;
 
   tmp = (unsigned char *) buf;
@@ -411,7 +366,9 @@ void put_uint32(void *buf, uint32_t value) {
   tmp[3] = (value >>= 8) & 0xff;
 }
 
-void put_uint64(void *buf, uint64_t value) {
+void
+put_uint64_le(void *buf,
+              uint64_t value) {
   unsigned char *tmp;
 
   tmp = (unsigned char *) buf;
@@ -426,56 +383,111 @@ void put_uint64(void *buf, uint64_t value) {
   tmp[7] = (value >>= 8) & 0xff;
 }
 
+void
+put_uint16_be(void *buf,
+              uint16_t value) {
+  unsigned char *tmp;
+
+  tmp = (unsigned char *) buf;
+
+  tmp[1] = value & 0xff;
+  tmp[0] = (value >>= 8) & 0xff;
+}
+
+void
+put_uint32_be(void *buf,
+              uint32_t value) {
+  unsigned char *tmp;
+
+  tmp = (unsigned char *) buf;
+
+  tmp[3] = value & 0xff;
+  tmp[2] = (value >>= 8) & 0xff;
+  tmp[1] = (value >>= 8) & 0xff;
+  tmp[0] = (value >>= 8) & 0xff;
+}
+
+void
+put_uint64_be(void *buf,
+              uint64_t value) {
+  unsigned char *tmp;
+
+  tmp = (unsigned char *) buf;
+
+  tmp[7] = value & 0xff;
+  tmp[6] = (value >>= 8) & 0xff;
+  tmp[5] = (value >>= 8) & 0xff;
+  tmp[4] = (value >>= 8) & 0xff;
+  tmp[3] = (value >>= 8) & 0xff;
+  tmp[2] = (value >>= 8) & 0xff;
+  tmp[1] = (value >>= 8) & 0xff;
+  tmp[0] = (value >>= 8) & 0xff;
+}
+
 /*
- * Character map conversion stuff
- */
+   Character map conversion stuff
+*/
 
 typedef struct {
   iconv_t ict_from_utf8, ict_to_utf8;
   char *charset;
 } kax_conv_t;
 
-static kax_conv_t *kax_convs = NULL;
-static int num_kax_convs = 0;
+static vector<kax_conv_t> kax_convs;
 int cc_local_utf8 = -1;
 
-int add_kax_conv(const char *charset, iconv_t ict_from, iconv_t ict_to) {
-  kax_conv_t *c;
+int
+add_kax_conv(const char *charset,
+             iconv_t ict_from,
+             iconv_t ict_to) {
+  kax_conv_t c;
   int i;
 
-  for (i = 0; i < num_kax_convs; i++)
+  for (i = 0; i < kax_convs.size(); i++)
     if (!strcmp(kax_convs[i].charset, charset))
       return i;
 
-  kax_convs = (kax_conv_t *)saferealloc(kax_convs, (num_kax_convs + 1) *
-                                        sizeof(kax_conv_t));
-  c = &kax_convs[num_kax_convs];
-  c->charset = safestrdup(charset);
-  c->ict_from_utf8 = ict_from;
-  c->ict_to_utf8 = ict_to;
-  num_kax_convs++;
+  c.charset = safestrdup(charset);
+  c.ict_from_utf8 = ict_from;
+  c.ict_to_utf8 = ict_to;
+  kax_convs.push_back(c);
 
-  return num_kax_convs - 1;
+  return kax_convs.size() - 1;
 }
 
-int utf8_init(const char *charset) {
+string
+get_local_charset() {
+  string lc_charset;
+
+  setlocale(LC_CTYPE, "");
+#if defined(COMP_MINGW) || defined(COMP_MSC)
+  lc_charset = "CP" + to_string(GetACP());
+#elif defined(SYS_SOLARIS)
+  lc_charset = nl_langinfo(CODESET);
+  if (parse_int(lc_charset, i))
+    lc_charset = string("ISO") + lc_charset + string("-US");
+#else
+  lc_charset = nl_langinfo(CODESET);
+#endif
+
+  return lc_charset;
+}
+
+int
+utf8_init(const string &charset) {
   string lc_charset;
   iconv_t ict_from_utf8, ict_to_utf8;
   int i;
 
-  if ((charset == NULL) || (*charset == 0)) {
-    setlocale(LC_CTYPE, "");
-#if defined(COMP_MINGW)
-    lc_charset = "CP" + to_string(GetACP());
-#else
-    lc_charset = nl_langinfo(CODESET);
-#endif
-    if ((lc_charset == "UTF8") || (lc_charset == "UTF-8"))
-      return -1;
-  } else
+  if (charset == "")
+    lc_charset = get_local_charset();
+  else
     lc_charset = charset;
 
-  for (i = 0; i < num_kax_convs; i++)
+  if ((lc_charset == "UTF8") || (lc_charset == "UTF-8"))
+    return -1;
+
+  for (i = 0; i < kax_convs.size(); i++)
     if (kax_convs[i].charset == lc_charset)
       return i;
 
@@ -498,29 +510,32 @@ int utf8_init(const char *charset) {
   return add_kax_conv(lc_charset.c_str(), ict_from_utf8, ict_to_utf8);
 }
 
-void utf8_done() {
+void
+utf8_done() {
   int i;
 
-  for (i = 0; i < num_kax_convs; i++) {
+  for (i = 0; i < kax_convs.size(); i++) {
     if (kax_convs[i].ict_from_utf8 != (iconv_t)(-1))
       iconv_close(kax_convs[i].ict_from_utf8);
     if (kax_convs[i].ict_to_utf8 != (iconv_t)(-1))
       iconv_close(kax_convs[i].ict_to_utf8);
     safefree(kax_convs[i].charset);
   }
-  if (kax_convs != NULL)
-    safefree(kax_convs);
+  kax_convs.clear();
 }
 
-static char *convert_charset(iconv_t ict, const char *src) {
+static string
+convert_charset(iconv_t ict,
+                const string &src) {
   char *dst, *psrc, *pdst, *srccopy;
   size_t lsrc, ldst;
   int len;
+  string result;
 
   if (ict == (iconv_t)(-1))
-    return safestrdup(src);
+    return src;
 
-  len = strlen(src) * 4;
+  len = src.length() * 4;
   dst = (char *)safemalloc(len + 1);
   memset(dst, 0, len + 1);
 
@@ -532,95 +547,129 @@ static char *convert_charset(iconv_t ict, const char *src) {
   pdst = dst;
   iconv(ict, (ICONV_CONST char **)&psrc, &lsrc, &pdst, &ldst);
   safefree(srccopy);
+  result = dst;
+  safefree(dst);
 
-  return dst;
+  return result;
 }
 
-char *to_utf8(int handle, const char *local) {
-  char *copy;
+string
+to_utf8(int handle,
+        const string &local) {
+  string s;
 
-  if (handle == -1) {
-    copy = safestrdup(local);
-    return copy;
-  }
+  if (handle == -1)
+    return local;
 
-  if (handle >= num_kax_convs)
+  if (handle >= kax_convs.size())
     die("common.cpp/to_utf8(): Invalid conversion handle %d (num: %d).",
-        handle, num_kax_convs);
+        handle, kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_to_utf8, local);
 }
 
-string &to_utf8(int handle, string &local) {
-  char *cutf8;
+string
+from_utf8(int handle,
+          const string &utf8) {
+  string s;
 
-  cutf8 = to_utf8(handle, local.c_str());
-  local = cutf8;
-  safefree(cutf8);
+  if (handle == -1)
+    return utf8;
 
-  return local;
-}
-
-char *from_utf8(int handle, const char *utf8) {
-  char *copy;
-
-  if (handle == -1) {
-    copy = safestrdup(utf8);
-    return copy;
-  }
-
-  if (handle >= num_kax_convs)
+  if (handle >= kax_convs.size())
     die("common.cpp/from_utf8(): Invalid conversion handle %d (num: %d).",
-        handle, num_kax_convs);
+        handle, kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_from_utf8, utf8);
 }
 
-string &from_utf8(int handle, string &utf8) {
-  char *clocal;
-
-  clocal = from_utf8(handle, utf8.c_str());
-  utf8 = clocal;
-  safefree(clocal);
-
-  return utf8;
+void
+init_cc_stdio() {
+  cc_stdio = utf8_init(get_local_charset());
 }
 
 /*
- * Random unique uint32_t numbers
- */
+   Random unique uint32_t numbers
+*/
 
-static vector<uint32_t> ru_numbers;
+static vector<uint32_t> ru_numbers[4];
+static bool random_seeded = false;
 
-void clear_list_of_unique_uint32() {
-  ru_numbers.clear();
+void
+clear_list_of_unique_uint32(unique_id_category_e category) {
+  int i;
+
+  assert((category >= UNIQUE_ALL_IDS) && (category <= UNIQUE_ATTACHMENT_IDS));
+  if (category == UNIQUE_ALL_IDS)
+    for (i = 0; i < 4; i++)
+      clear_list_of_unique_uint32((unique_id_category_e)i);
+  else
+    ru_numbers[category].clear();
 }
 
-bool is_unique_uint32(uint32_t number) {
+bool
+is_unique_uint32(uint32_t number,
+                 unique_id_category_e category) {
   int i;
 
+  assert((category >= UNIQUE_TRACK_IDS) &&
+         (category <= UNIQUE_ATTACHMENT_IDS));
+
   if (hack_engaged(ENGAGE_NO_VARIABLE_DATA))
     return true;
-  for (i = 0; i < ru_numbers.size(); i++)
-    if (ru_numbers[i] == number)
+  for (i = 0; i < ru_numbers[category].size(); i++)
+    if (ru_numbers[category][i] == number)
       return false;
 
   return true;
 }
 
-void add_unique_uint32(uint32_t number) {
+void
+add_unique_uint32(uint32_t number,
+                  unique_id_category_e category) {
+  assert((category >= UNIQUE_TRACK_IDS) &&
+         (category <= UNIQUE_ATTACHMENT_IDS));
+
   if (hack_engaged(ENGAGE_NO_VARIABLE_DATA))
-    ru_numbers.push_back(ru_numbers.size() + 1);
+    ru_numbers[category].push_back(ru_numbers[category].size() + 1);
   else
-    ru_numbers.push_back(number);
+    ru_numbers[category].push_back(number);
 }
 
-uint32_t create_unique_uint32() {
+bool
+remove_unique_uint32(uint32_t number,
+                     unique_id_category_e category) {
+  vector<uint32_t>::iterator dit;
+
+  assert((category >= UNIQUE_TRACK_IDS) &&
+         (category <= UNIQUE_ATTACHMENT_IDS));
+
+  if (hack_engaged(ENGAGE_NO_VARIABLE_DATA))
+    return true;
+  foreach(dit, ru_numbers[category])
+    if (*dit == number) {
+      ru_numbers[category].erase(dit);
+      return true;
+    }
+
+  return false;
+}
+
+uint32_t
+create_unique_uint32(unique_id_category_e category) {
   uint32_t rnumber, half;
 
+  assert((category >= UNIQUE_TRACK_IDS) &&
+         (category <= UNIQUE_ATTACHMENT_IDS));
+
   if (hack_engaged(ENGAGE_NO_VARIABLE_DATA)) {
-    ru_numbers.push_back(ru_numbers.size() + 1);
-    return ru_numbers.size();
+    ru_numbers[category].push_back(ru_numbers[category].size() + 1);
+    return ru_numbers[category].size();
+  }
+
+  if (!random_seeded) {
+    srand(time(NULL));
+    random_seeded = true;
   }
 
   do {
@@ -628,51 +677,31 @@ uint32_t create_unique_uint32() {
     rnumber = half;
     half = (uint32_t)(65535.0 * rand() / RAND_MAX);
     rnumber |= (half << 16);
-  } while ((rnumber == 0) || !is_unique_uint32(rnumber));
-  add_unique_uint32(rnumber);
+  } while ((rnumber == 0) || !is_unique_uint32(rnumber, category));
+  add_unique_uint32(rnumber, category);
 
   return rnumber;
 }
 
 /*
- * Miscellaneous stuff
- */
+   Miscellaneous stuff
+*/
 
-char *_safestrdup(const char *s, const char *file, int line) {
-  char *copy;
+static uint64_t _safedupped = 0;
+static uint64_t _safemalloced = 0;
 
-  if (s == NULL)
-    return NULL;
-
-  copy = strdup(s);
-  if (copy == NULL)
-    die("common.cpp/safestrdup() called from file %s, line %d: strdup() "
-        "returned NULL for '%s'.", file, line, s);
-
-  return copy;
-}
-
-unsigned char *_safestrdup(const unsigned char *s, const char *file,
-                           int line) {
-  char *copy;
-
-  if (s == NULL)
-    return NULL;
-
-  copy = strdup((const char *)s);
-  if (copy == NULL)
-    die("common.cpp/safestrdup() called from file %s, line %d: strdup() "
-        "returned NULL for '%s'.", file, line, s);
-
-  return (unsigned char *)copy;
-}
-
-void *_safememdup(const void *s, size_t size, const char *file, int line) {
+void *
+_safememdup(const void *s,
+            size_t size,
+            const char *file,
+            int line) {
   void *copy;
 
   if (s == NULL)
     return NULL;
 
+  _safedupped += size;
+
   copy = malloc(size);
   if (copy == NULL)
     die("common.cpp/safememdup() called from file %s, line %d: malloc() "
@@ -682,9 +711,14 @@ void *_safememdup(const void *s, size_t size, const char *file, int line) {
   return copy;
 }
 
-void *_safemalloc(size_t size, const char *file, int line) {
+void *
+_safemalloc(size_t size,
+            const char *file,
+            int line) {
   void *mem;
 
+  _safemalloced += size;
+
   mem = malloc(size);
   if (mem == NULL)
     die("common.cpp/safemalloc() called from file %s, line %d: malloc() "
@@ -693,7 +727,11 @@ void *_safemalloc(size_t size, const char *file, int line) {
   return mem;
 }
 
-void *_saferealloc(void *mem, size_t size, const char *file, int line) {
+void *
+_saferealloc(void *mem,
+             size_t size,
+             const char *file,
+             int line) {
   mem = realloc(mem, size);
   if (mem == NULL)
     die("common.cpp/saferealloc() called from file %s, line %d: realloc() "
@@ -702,11 +740,26 @@ void *_saferealloc(void *mem, size_t size, const char *file, int line) {
   return mem;
 }
 
-/*
- * standard string processing
- */
+void
+safefree(void *p) {
+  if (p != NULL)
+    free(p);
+}
 
-vector<string> split(const char *src, const char *pattern, int max_num) {
+void
+dump_malloc_report() {
+  mxinfo("%llu bytes malloced, %llu bytes duplicated\n",
+         _safemalloced - _safedupped, _safedupped);
+}
+
+/*
+   standard string processing
+*/
+
+vector<string>
+split(const char *src,
+      const char *pattern,
+      int max_num) {
   int num, i, plen;
   char *copy, *p1, *p2;
   vector<string> v;
@@ -731,7 +784,9 @@ vector<string> split(const char *src, const char *pattern, int max_num) {
   return v;
 }
 
-string join(const char *pattern, vector<string> &strings) {
+string
+join(const char *pattern,
+     vector<string> &strings) {
   string dst;
   uint32_t i;
 
@@ -746,7 +801,9 @@ string join(const char *pattern, vector<string> &strings) {
   return dst;
 }
 
-void strip(string &s, bool newlines) {
+void
+strip(string &s,
+      bool newlines) {
   int i, len;
   const char *c;
 
@@ -777,17 +834,21 @@ void strip(string &s, bool newlines) {
     s.erase(len - i, i);
 }
 
-void strip(vector<string> &v, bool newlines) {
+void
+strip(vector<string> &v,
+      bool newlines) {
   int i;
 
   for (i = 0; i < v.size(); i++)
     strip(v[i], newlines);
 }
 
-string escape(const char *src) {
+string
+escape(const string &source) {
   string dst;
+  string::const_iterator src;
 
-  while (*src != 0) {
+  for (src = source.begin(); src < source.end(); src++) {
     if (*src == '\\')
       dst += "\\\\";
     else if (*src == '"')
@@ -796,20 +857,21 @@ string escape(const char *src) {
       dst += "\\s";
     else
       dst += *src;
-    src++;
   }
 
   return dst;
 }
 
-string unescape(const char *src) {
+string
+unescape(const string &source) {
   string dst;
-  const char *next_char;
+  string::const_iterator src, next_char;
 
+  src = source.begin();
   next_char = src + 1;
-  while (*src != 0) {
+  while (src != source.end()) {
     if (*src == '\\') {
-      if (*next_char == 0)      // This is an error...
+      if (next_char == source.end()) // This is an error...
         dst += '\\';
       else {
         if (*next_char == '2')
@@ -829,10 +891,13 @@ string unescape(const char *src) {
   return dst;
 }
 
-string escape_xml(const char *src) {
+string
+escape_xml(const string &source) {
   string dst;
+  string::const_iterator src;
 
-  while (*src != 0) {
+  src = source.begin();
+  while (src != source.end()) {
     if (*src == '&')
       dst += "&amp;";
     else if (*src == '>')
@@ -847,27 +912,66 @@ string escape_xml(const char *src) {
   return dst;
 }
 
-bool starts_with(const string &s, const char *start) {
-  return strncmp(s.c_str(), start, strlen(start)) == 0;
+bool
+starts_with(const string &s,
+            const char *start,
+            int maxlen) {
+  int len, slen;
+
+  slen = strlen(start);
+  if (maxlen == -1)
+    len = slen;
+  else
+    len = maxlen < slen ? maxlen : slen;
+
+  return strncmp(s.c_str(), start, slen) == 0;
 }
 
-bool starts_with(const string &s, const string &start) {
-  return strncmp(s.c_str(), start.c_str(), start.length()) == 0;
+bool
+starts_with_case(const string &s,
+                 const char *start,
+                 int maxlen) {
+  int len, slen;
+
+  slen = strlen(start);
+  if (maxlen == -1)
+    len = slen;
+  else
+    len = maxlen < slen ? maxlen : slen;
+
+  return strncasecmp(s.c_str(), start, slen) == 0;
 }
 
-bool starts_with_case(const string &s, const char *start) {
-  return strncasecmp(s.c_str(), start, strlen(start)) == 0;
+string
+upcase(const string &s) {
+  string dst;
+  int i;
+
+  dst.reserve(s.size());
+  for (i = 0; i < s.size(); i++)
+    dst += toupper(s[i]);
+
+  return dst;
 }
 
-bool starts_with_case(const string &s, const string &start) {
-  return strncasecmp(s.c_str(), start.c_str(), start.length()) == 0;
+string
+downcase(const string &s) {
+  string dst;
+  int i;
+
+  dst.reserve(s.size());
+  for (i = 0; i < s.size(); i++)
+    dst += tolower(s[i]);
+
+  return dst;
 }
 
 /*
- * Integer parsing
- */
+   Integer parsing
+*/
 
-uint32_t round_to_nearest_pow2(uint32_t value) {
+uint32_t
+round_to_nearest_pow2(uint32_t value) {
   int64_t best_value, test_value;
 
   best_value = 0;
@@ -881,7 +985,9 @@ uint32_t round_to_nearest_pow2(uint32_t value) {
   return best_value;
 }
 
-bool parse_int(const char *s, int64_t &value) {
+bool
+parse_int(const char *s,
+          int64_t &value) {
   const char *p;
   int sign;
 
@@ -907,7 +1013,9 @@ bool parse_int(const char *s, int64_t &value) {
   return true;
 }
 
-bool parse_int(const char *s, int &value) {
+bool
+parse_int(const char *s,
+          int &value) {
   int64_t tmp;
   bool result;
 
@@ -917,7 +1025,9 @@ bool parse_int(const char *s, int &value) {
   return result;
 }
 
-bool parse_double(const char *s, double &value) {
+bool
+parse_double(const char *s,
+             double &value) {
   char *endptr;
   string old_locale;
   bool ok;
@@ -940,7 +1050,8 @@ bool parse_double(const char *s, double &value) {
   return ok;
 }
 
-string to_string(int64_t i) {
+string
+to_string(int64_t i) {
   char buf[100];
 
   sprintf(buf, "%lld", i);
@@ -950,8 +1061,8 @@ string to_string(int64_t i) {
 
 #ifdef DEBUG
 /*
- * debugging stuff
- */
+   debugging stuff
+*/
 
 debug_c debug_facility;
 
@@ -965,7 +1076,8 @@ debug_c::~debug_c() {
   }
 }
 
-void debug_c::enter(const char *label) {
+void
+debug_c::enter(const char *label) {
   int i;
   debug_data_t *entry;
 #if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
@@ -996,7 +1108,8 @@ void debug_c::enter(const char *label) {
 #endif
 }
 
-void debug_c::leave(const char *label) {
+void
+debug_c::leave(const char *label) {
   int i;
   debug_data_t *entry;
 #if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
@@ -1029,7 +1142,8 @@ void debug_c::leave(const char *label) {
   entry->entered_at = 0;
 }
 
-void debug_c::add_packetizer(void *ptzr) {
+void
+debug_c::add_packetizer(void *ptzr) {
   int i;
 
   for (i = 0; i < packetizers.size(); i++)
@@ -1039,9 +1153,10 @@ void debug_c::add_packetizer(void *ptzr) {
   packetizers.push_back((generic_packetizer_c *)ptzr);
 }
 
-void debug_c::dump_info() {
-  int i;
+void
+debug_c::dump_info() {
 #if defined(SYS_UNIX) || defined(SYS_APPLE)
+  int i;
   debug_data_t *entry;
   uint64_t diff_calls, diff_time;
 
@@ -1071,10 +1186,12 @@ void debug_c::dump_info() {
 #endif // DEBUG
 
 /*
- * Other related news
- */
+   Other related news
+*/
 
-void fix_format(const char *fmt, string &new_fmt) {
+void
+fix_format(const char *fmt,
+           string &new_fmt) {
 #if defined(COMP_MINGW) || defined(COMP_MSC)
   int i, len;
   bool state;
@@ -1113,7 +1230,12 @@ void fix_format(const char *fmt, string &new_fmt) {
 #endif
 }
 
-void mxprint(void *stream, const char *fmt, ...) {
+int cc_stdio = -1;
+
+void
+mxprint(void *stream,
+        const char *fmt,
+        ...) {
   va_list ap;
   string new_fmt;
 
@@ -1124,7 +1246,10 @@ void mxprint(void *stream, const char *fmt, ...) {
   va_end(ap);
 }
 
-void mxprints(char *dst, const char *fmt, ...) {
+void
+mxprints(char *dst,
+         const char *fmt,
+         ...) {
   va_list ap;
   string new_fmt;
 
@@ -1134,8 +1259,11 @@ void mxprints(char *dst, const char *fmt, ...) {
   va_end(ap);
 }
 
-static void mxmsg(int level, const char *fmt, va_list &ap) {
-  string new_fmt;
+static void
+mxmsg(int level,
+      const char *fmt,
+      va_list &ap) {
+  string new_fmt, output;
   bool nl;
   FILE *stream;
   char *prefix;
@@ -1164,11 +1292,15 @@ static void mxmsg(int level, const char *fmt, va_list &ap) {
   if (prefix != NULL)
     fprintf(stream, prefix);
 
-  vfprintf(stream, new_fmt.c_str(), ap);
+  output = from_utf8(cc_stdio, mxvsprintf(new_fmt.c_str(), ap));
+  fputs(output.c_str(), stream);
   fflush(stream);
 }
 
-void mxverb(int level, const char *fmt, ...) {
+void
+mxverb(int level,
+       const char *fmt,
+       ...) {
   va_list ap;
 
   if (verbose < level)
@@ -1179,7 +1311,9 @@ void mxverb(int level, const char *fmt, ...) {
   va_end(ap);
 }
 
-void mxinfo(const char *fmt, ...) {
+void
+mxinfo(const char *fmt,
+       ...) {
   va_list ap;
 
   va_start(ap, fmt);
@@ -1189,7 +1323,9 @@ void mxinfo(const char *fmt, ...) {
 
 static bool warning_issued = false;
 
-void mxwarn(const char *fmt, ...) {
+void
+mxwarn(const char *fmt,
+       ...) {
   va_list ap;
 
   if (suppress_warnings)
@@ -1201,7 +1337,9 @@ void mxwarn(const char *fmt, ...) {
   warning_issued = true;
 }
 
-void mxerror(const char *fmt, ...) {
+void
+mxerror(const char *fmt,
+        ...) {
   va_list ap;
 
   va_start(ap, fmt);
@@ -1212,7 +1350,9 @@ void mxerror(const char *fmt, ...) {
   exit(2);
 }
 
-void mxdebug(const char *fmt, ...) {
+void
+mxdebug(const char *fmt,
+        ...) {
   va_list ap;
 
   va_start(ap, fmt);
@@ -1220,7 +1360,8 @@ void mxdebug(const char *fmt, ...) {
   va_end(ap);
 }
 
-void mxexit(int code) {
+void
+mxexit(int code) {
   if (code != -1)
     exit(code);
 
@@ -1230,7 +1371,9 @@ void mxexit(int code) {
   exit(0);
 }
 
-int get_arg_len(const char *fmt, ...) {
+int
+get_arg_len(const char *fmt,
+            ...) {
   int size;
   va_list ap;
 
@@ -1241,7 +1384,9 @@ int get_arg_len(const char *fmt, ...) {
   return size;
 }
 
-int get_varg_len(const char *fmt, va_list ap) {
+int
+get_varg_len(const char *fmt,
+             va_list ap) {
   int size, result;
   char *dst;
 
@@ -1261,14 +1406,14 @@ int get_varg_len(const char *fmt, va_list ap) {
   return -1;
 }
 
-string mxsprintf(const char *fmt, ...) {
-  va_list ap;
+string
+mxvsprintf(const char *fmt,
+           va_list &ap) {
   string new_fmt, dst;
   char *new_string;
   int len;
 
   fix_format(fmt, new_fmt);
-  va_start(ap, fmt);
   len = get_varg_len(new_fmt.c_str(), ap);
   new_string = (char *)safemalloc(len + 1);
   vsprintf(new_string, new_fmt.c_str(), ap);
@@ -1279,55 +1424,272 @@ string mxsprintf(const char *fmt, ...) {
   return dst;
 }
 
-static const char *mosu_hacks[] = {
-  ENGAGE_SPACE_AFTER_CHAPTERS,
-  ENGAGE_NO_CHAPTERS_IN_META_SEEK,
-  ENGAGE_NO_META_SEEK,
-  ENGAGE_LACING_XIPH,
-  ENGAGE_LACING_EBML,
-  ENGAGE_NATIVE_BFRAMES,
-  ENGAGE_NO_VARIABLE_DATA,
-  ENGAGE_NO_DEFAULT_HEADER_VALUES,
-  ENGAGE_FORCE_PASSTHROUGH_PACKETIZER,
-  ENGAGE_SKIP_BIG_VOBSUBS,
-  NULL
-};
-static vector<const char *> engaged_hacks;
+string
+mxsprintf(const char *fmt,
+          ...) {
+  va_list ap;
+  string result;
 
-bool hack_engaged(const char *hack) {
-  uint32_t i;
+  va_start(ap, fmt);
+  result = mxvsprintf(fmt, ap);
+  va_end(ap);
 
-  if (hack == NULL)
-    return false;
-  for (i = 0; i < engaged_hacks.size(); i++)
-    if (!strcmp(engaged_hacks[i], hack))
-      return true;
+  return result;
+}
 
+/** \brief Platform independant version of sscanf
+
+   This is a platform independant version of sscanf. It first fixes the format
+   string (\see fix_format) and then calls sscanf.
+
+   \param str The string to parse
+   \param fmt The format string
+   \returns The number of elements assigned
+*/
+int
+mxsscanf(const string &str,
+         const char *fmt,
+         ...) {
+  va_list ap;
+  string new_fmt;
+  int result;
+
+  mxverb(5, "mxsscanf: str: %s /// fmt: %s\n", str.c_str(), fmt);
+  fix_format(fmt, new_fmt);
+  va_start(ap, fmt);
+  result = vsscanf(str.c_str(), new_fmt.c_str(), ap);
+  va_end(ap);
+
+  return result;
+}
+
+void
+mxhexdump(int level,
+          const unsigned char *buffer,
+          int length) {
+  int i, j;
+  char output[24];
+
+  if (verbose < level)
+    return;
+  j = 0;
+  for (i = 0; i < length; i++) {
+    if ((i % 16) == 0) {
+      if (i > 0) {
+        output[j] = 0;
+        mxinfo("%s\n", output);
+        j = 0;
+      }
+      mxinfo("%08x  ", i);
+
+    } else if ((i % 8) == 0) {
+      mxinfo(" ");
+      output[j] = ' ';
+      j++;
+
+    }
+    if ((buffer[i] >= 32) && (buffer[i] < 128))
+      output[j] = buffer[i];
+    else
+      output[j] = '.';
+    j++;
+    mxinfo("%02x ", buffer[i]);
+  }
+  while ((i % 16) != 0) {
+    if ((i % 8) == 0)
+      mxinfo(" ");
+    mxinfo("   ");
+    i++;
+  }
+  output[j] = 0;
+  mxinfo("%s\n", output);
+}
+
+const char *timecode_parser_error = NULL;
+
+static bool
+set_tcp_error(const char *error) {
+  timecode_parser_error = error;
   return false;
 }
 
-void engage_hacks(const char *hacks) {
-  vector<string> engage_args;
-  int aidx, hidx;
-  bool valid_hack;
+bool
+parse_timecode(const char *src,
+               int64_t *timecode) {
+  // Recognized format:
+  // HH:MM:SS and HH:MM:SS.nnn with up to nine 'n' for ns precision
+  //              012345678901...
+  int h, m, s, n, i;
+  char format[10];
 
-  engage_args = split(hacks, ",");
-  for (aidx = 0; aidx < engage_args.size(); aidx++)
-    if (engage_args[aidx] == "list") {
-      mxinfo("Valid hacks are:\n");
-      for (hidx = 0; mosu_hacks[hidx] != NULL; hidx++)
-        mxinfo("%s\n", mosu_hacks[hidx]);
-      mxexit(0);
-    }
-  for (aidx = 0; aidx < engage_args.size(); aidx++) {
-    valid_hack = false;
-    for (hidx = 0; mosu_hacks[hidx] != NULL; hidx++)
-      if (engage_args[aidx] == mosu_hacks[hidx]) {
-        valid_hack = true;
-        engaged_hacks.push_back(mosu_hacks[hidx]);
-        break;
-      }
-    if (!valid_hack)
-      mxerror("'%s' is not a valid hack.\n", engage_args[aidx].c_str());
-  }
+  if ((strlen(src) < 8) || (strlen(src) > 18) || (src[2] != ':') ||
+      (src[5] != ':'))
+    return set_tcp_error("Invalid format");
+  if (sscanf(src, "%02d:%02d:%02d", &h, &m, &s) != 3)
+    return set_tcp_error("Invalid format (non-numbers encountered)");
+  if (m > 59)
+    return set_tcp_error("Invalid minute");
+  if (s > 59)
+    return set_tcp_error("Invalid second");
+  if (strlen(src) > 9) {
+    if (src[8] != '.')
+      return set_tcp_error("Invalid format (expected a dot '.' after the "
+                           "seconds)");
+    sprintf(format, "%%0%dd", strlen(src) - 8);
+    if (sscanf(&src[9], format, &n) != 1)
+      return set_tcp_error("Invalid format (non-numbers encountered)");
+    for (i = strlen(src); i < 18; i++)
+      n *= 10;
+  } else
+    n = 0;
+  if (timecode != NULL)
+    *timecode = ((int64_t)h * 60 * 60 + (int64_t)m * 60 + (int64_t)s) *
+      1000000000 + n;
+  timecode_parser_error = "no error";
+  return true;
 }
+
+/** \brief Reads command line arguments from a file
+
+   Each line contains exactly one command line argument or a
+   comment. Arguments are converted to UTF-8 and appended to the array
+   \c args.
+*/
+static void
+read_args_from_file(vector<string> &args,
+                    const string &filename) {
+  mm_text_io_c *mm_io;
+  string buffer;
+  bool skip_next;
+
+  mm_io = NULL;
+  try {
+    mm_io = new mm_text_io_c(new mm_file_io_c(filename));
+  } catch (...) {
+    mxerror(_("The file '%s' could not be opened for reading command line "
+              "arguments."), filename.c_str());
+  }
+
+  skip_next = false;
+  while (!mm_io->eof() && mm_io->getline2(buffer)) {
+    if (skip_next) {
+      skip_next = false;
+      continue;
+    }
+    strip(buffer);
+
+    if (buffer == "#EMPTY#") {
+      args.push_back("");
+      continue;
+    }
+
+    if ((buffer[0] == '#') || (buffer[0] == 0))
+      continue;
+
+    if (buffer == "--command-line-charset") {
+      skip_next = true;
+      continue;
+    }
+    args.push_back(buffer);
+  }
+
+  delete mm_io;
+}
+
+/** \brief Expand the command line parameters
+
+   Takes each command line paramter, converts it to UTF-8, and reads more
+   commands from command files if the argument starts with '@'. Puts all
+   arguments into a new array.
+   On Windows it uses the \c GetCommandLineW() function. That way it can
+   also handle multi-byte input like Japanese file names.
+
+   \param argc The number of arguments. This is the same argument that
+     \c main normally receives.
+   \param argv The arguments themselves. This is the same argument that
+     \c main normally receives.
+   \return An array of strings converted to UTF-8 containing all the
+     command line arguments and any arguments read from option files.
+*/
+#if !defined(SYS_WINDOWS)
+vector<string>
+command_line_utf8(int argc,
+                  char **argv) {
+  int i, cc_command_line;
+  vector<string> args;
+
+  cc_command_line = cc_local_utf8;
+
+  for (i = 1; i < argc; i++)
+    if (argv[i][0] == '@')
+      read_args_from_file(args, &argv[i][1]);
+    else {
+      if (!strcmp(argv[i], "--command-line-charset")) {
+        if ((i + 1) == argc)
+          mxerror(_("'--command-line-charset' is missing its argument.\n"));
+        cc_command_line = utf8_init(argv[i + 1] == NULL ? "" : argv[i + 1]);
+        i++;
+      } else
+        args.push_back(to_utf8(cc_command_line, argv[i]));
+    }
+
+  return args;
+}
+
+#else  // !defined(SYS_WINDOWS)
+
+vector<string>
+command_line_utf8(int,
+                  char **) {
+  vector<string> args;
+  wchar_t *p;
+  string utf8;
+  bool quoted, ignore_me, skip_first;
+
+  p = GetCommandLineW();
+  quoted = false;
+  ignore_me = true;
+  skip_first = true;
+  while (*p != 0) {
+    if (*p == L'"')
+      quoted = !quoted;
+    else if (*p == L' ') {
+      if (quoted)
+        utf8 += ' ';
+      else if (!ignore_me) {
+        if (!skip_first) {
+          if (utf8[0] == '@')
+            read_args_from_file(args, utf8.substr(1).c_str());
+          else
+            args.push_back(utf8);
+        }
+        skip_first = false;
+        ignore_me = true;
+        utf8.clear();
+      }
+    } else {
+      ignore_me = false;
+      if (*p < 0x80)
+        utf8 += (char)*p;
+      else if (*p < 0x800) {
+        utf8 += (char)(0xc0 | (*p >> 6));
+        utf8 += (char)(0x80 | (*p & 0x3f));
+      } else {
+        utf8 += (char)(0xe0 | (*p >> 12));
+        utf8 += (char)(0x80 | ((*p >> 6) & 0x3f));
+        utf8 += (char)(0x80 | (*p & 0x3f));
+      }
+    }
+
+    ++p;
+  }
+  if (!ignore_me && !skip_first) {
+    if (utf8[0] == '@')
+      read_args_from_file(args, utf8.substr(1));
+    else
+      args.push_back(utf8);
+  }
+
+  return args;
+}
+#endif // !defined(SYS_WINDOWS)
