@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  r_srt.cpp
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: r_srt.cpp 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   Subripper subtitle reader
 
-/*!
-    \file
-    \version $Id: r_srt.cpp 1434 2004-01-22 22:36:05Z mosu $
-    \brief Subripper subtitle reader
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include <ctype.h>
@@ -24,7 +19,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "mkvmerge.h"
 #include "pr_generic.h"
 #include "r_srt.h"
 #include "subtitles.h"
@@ -46,52 +40,61 @@ using namespace std;
                            istimecode(s + 17))
 
 int
-srt_reader_c::probe_file(mm_text_io_c *mm_io,
+srt_reader_c::probe_file(mm_text_io_c *io,
                          int64_t) {
   string s;
   int64_t dummy;
 
   try {
-    mm_io->setFilePointer(0, seek_beginning);
-    s = mm_io->getline();
-    if (!parse_int(s.c_str(), dummy))
+    io->setFilePointer(0, seek_beginning);
+    s = io->getline();
+    strip(s);
+    if (!parse_int(s, dummy))
       return 0;
-    s = mm_io->getline();
+    s = io->getline();
     if ((s.length() < 29) || !issrttimecode(s.c_str()))
       return 0;
-    s = mm_io->getline();
-    mm_io->setFilePointer(0, seek_beginning);
-  } catch (exception &ex) {
+    s = io->getline();
+    io->setFilePointer(0, seek_beginning);
+  } catch (...) {
     return 0;
   }
   return 1;
 }
 
-srt_reader_c::srt_reader_c(track_info_c *nti)
+srt_reader_c::srt_reader_c(track_info_c &_ti)
   throw (error_c):
-  generic_reader_c(nti) {
-  bool is_utf8;
+  generic_reader_c(_ti) {
 
-  act_wchar = 0;
   try {
-    mm_io = new mm_text_io_c(ti->fname);
-    if (!srt_reader_c::probe_file(mm_io, 0))
+    io = new mm_text_io_c(new mm_file_io_c(ti.fname));
+    if (!srt_reader_c::probe_file(io, 0))
       throw error_c("srt_reader: Source is not a valid SRT file.");
-    ti->id = 0;                 // ID for this track.
-    is_utf8 = mm_io->get_byte_order() != BO_NONE;
-    textsubs_packetizer = new textsubs_packetizer_c(this, MKV_S_TEXTUTF8, NULL,
-                                                    0, true, is_utf8, ti);
-  } catch (exception &ex) {
+    ti.id = 0;                 // ID for this track.
+  } catch (...) {
     throw error_c("srt_reader: Could not open the source file.");
   }
   if (verbose)
-    mxinfo("Using SRT subtitle reader for %s.\n+-> Using "
-           "text subtitle output module for subtitles.\n", ti->fname);
+    mxinfo(FMT_FN "Using the SRT subtitle reader.\n", ti.fname.c_str());
+  parse_file();
 }
 
 srt_reader_c::~srt_reader_c() {
-  if (textsubs_packetizer != NULL)
-    delete textsubs_packetizer;
+  delete io;
+}
+
+void
+srt_reader_c::create_packetizer(int64_t) {
+  bool is_utf8;
+
+  if (NPTZR() != 0)
+    return;
+
+  is_utf8 = io->get_byte_order() != BO_NONE;
+  add_packetizer(new textsubs_packetizer_c(this, MKV_S_TEXTUTF8, NULL, 0,
+                                           true, is_utf8, ti));
+  mxinfo(FMT_TID "Using the text subtitle output module.\n", ti.fname.c_str(),
+         (int64_t)0);
 }
 
 #define STATE_INITIAL         0
@@ -99,22 +102,23 @@ srt_reader_c::~srt_reader_c() {
 #define STATE_SUBS_OR_NUMBER  2
 #define STATE_TIME            3
 
-int
-srt_reader_c::read(generic_packetizer_c *) {
-  int64_t start, end;
+void
+srt_reader_c::parse_file() {
+  int64_t start, end, previous_start;
   char *chunk;
-  subtitles_c subs;
   string s, subtitles;
   int state, i, line_number;
-  bool non_number_found;
+  bool non_number_found, timecode_warning_printed;
 
   start = 0;
   end = 0;
+  previous_start = 0;
+  timecode_warning_printed = false;
   state = STATE_INITIAL;
   line_number = 0;
   subtitles = "";
   while (1) {
-    if (!mm_io->getline2(s))
+    if (!io->getline2(s))
       break;
     line_number++;
     strip(s);
@@ -133,8 +137,8 @@ srt_reader_c::read(generic_packetizer_c *) {
       non_number_found = false;
       for (i = 0; i < s.length(); i++)
         if (!isdigit(s[i])) {
-          mxwarn("srt_reader: Error in line %d: expected subtitle number "
-                 "and found some text.\n", line_number);
+          mxwarn(FMT_FN "Error in line %d: expected subtitle number "
+                 "and found some text.\n", ti.fname.c_str(), line_number);
           non_number_found = true;
           break;
         }
@@ -144,9 +148,9 @@ srt_reader_c::read(generic_packetizer_c *) {
 
     } else if (state == STATE_TIME) {
       if ((s.length() < 29) || !issrttimecode(s.c_str())) {
-        mxwarn("srt_reader: Error in line %d: expected a SRT timecode "
+        mxwarn(FMT_FN "Error in line %d: expected a SRT timecode "
                "line but found something else. Aborting this file.\n",
-               line_number);
+               ti.fname.c_str(), line_number);
         break;
       }
 
@@ -175,6 +179,15 @@ srt_reader_c::read(generic_packetizer_c *) {
         atol(&chunk[23]) * 1000 + atol(&chunk[26]);
       end *= 1000000;
 
+      if (!timecode_warning_printed && (start < previous_start)) {
+        mxwarn(FMT_FN "Warning in line %d: The start timecode is smaller "
+               "than that of the previous entry. All entries from this file "
+               "will be sorted by their start time.\n", ti.fname.c_str(),
+               line_number);
+        timecode_warning_printed = true;
+      }
+      previous_start = start;
+
       safefree(chunk);
 
       subtitles = "";
@@ -208,39 +221,32 @@ srt_reader_c::read(generic_packetizer_c *) {
     subs.add(start, end, subtitles.c_str());
   }
 
-  if ((subs.check() != 0) && verbose)
-    mxwarn("srt_reader: The subtitle file seems to be "
-           "badly broken. The output file might not be playable "
-           "correctly.\n");
-  subs.process(textsubs_packetizer);
+  subs.sort();
+}
 
-  textsubs_packetizer->flush();
+file_status_e
+srt_reader_c::read(generic_packetizer_c *,
+                   bool) {
+  if (subs.empty())
+    return FILE_STATUS_DONE;
 
-  return 0;
+  subs.process((textsubs_packetizer_c *)PTZR0);
+
+  return subs.empty() ? FILE_STATUS_DONE : FILE_STATUS_MOREDATA;
 }
 
 int
-srt_reader_c::display_priority() {
-  return DISPLAYPRIORITY_LOW;
-}
+srt_reader_c::get_progress() {
+  int num_entries;
 
-static char wchar[] = "-\\|/-\\|/-";
-
-void
-srt_reader_c::display_progress(bool) {
-  mxinfo("working... %c\r", wchar[act_wchar]);
-  act_wchar++;
-  if (act_wchar == strlen(wchar))
-    act_wchar = 0;
-}
-
-void
-srt_reader_c::set_headers() {
-  textsubs_packetizer->set_headers();
+  num_entries = subs.get_num_entries();
+  if (num_entries == 0)
+    return 100;
+  return 100 * subs.get_num_processed() / num_entries;
 }
 
 void
 srt_reader_c::identify() {
   mxinfo("File '%s': container: SRT\nTrack ID 0: subtitles (SRT)\n",
-         ti->fname);
+         ti.fname.c_str());
 }
