@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  r_ssa.cpp
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: r_ssa.cpp 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   SSA/ASS subtitle parser
 
-/*!
-    \file
-    \version $Id: r_ssa.cpp 1434 2004-01-22 22:36:05Z mosu $
-    \brief SSA/ASS subtitle parser
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include <ctype.h>
@@ -26,88 +21,72 @@
 
 #include <algorithm>
 
-#include "mkvmerge.h"
 #include "pr_generic.h"
 #include "r_ssa.h"
-#include "subtitles.h"
 #include "matroska.h"
 
 using namespace std;
 
-class ssa_line_c {
-public:
-  char *line;
-  int64_t start, end;
-  int num;
-
-  bool operator < (const ssa_line_c &cmp) const;
-};
-
-bool
-ssa_line_c::operator < (const ssa_line_c &cmp) const {
-  return start < cmp.start;
-}
-
 int
-ssa_reader_c::probe_file(mm_text_io_c *mm_io,
+ssa_reader_c::probe_file(mm_text_io_c *io,
                          int64_t size) {
   string line;
 
   try {
-    mm_io->setFilePointer(0, seek_beginning);
-    line = mm_io->getline();
+    io->setFilePointer(0, seek_beginning);
+    line = io->getline();
     if (strcasecmp(line.c_str(), "[script info]"))
       return 0;
-    mm_io->setFilePointer(0, seek_beginning);
-  } catch (exception &ex) {
+    io->setFilePointer(0, seek_beginning);
+  } catch (...) {
     return 0;
   }
   return 1;
 }
 
-ssa_reader_c::ssa_reader_c(track_info_c *nti)
+ssa_reader_c::ssa_reader_c(track_info_c &_ti)
   throw (error_c):
-  generic_reader_c(nti) {
-  string line, global;
+  generic_reader_c(_ti) {
+  string line;
   int64_t old_pos;
   char section;
-  bool is_ass, sub_charset_found;
+  bool sub_charset_found;
   int i;
 
   is_ass = false;
   section = 0;
 
   try {
-    mm_io = new mm_text_io_c(ti->fname);
+    io = new mm_text_io_c(new mm_file_io_c(ti.fname));
 
-    if (!ssa_reader_c::probe_file(mm_io, 0))
+    if (!ssa_reader_c::probe_file(io, 0))
       throw error_c("ssa_reader: Source is not a valid SSA/ASS file.");
 
     sub_charset_found = false;
-    for (i = 0; i < ti->sub_charsets->size(); i++)
-      if ((*ti->sub_charsets)[i].id == 0) {
+    for (i = 0; i < ti.sub_charsets.size(); i++)
+      if ((ti.sub_charsets[i].id == 0) || (ti.sub_charsets[i].id == -1)) {
         sub_charset_found = true;
-        cc_utf8 = utf8_init((*ti->sub_charsets)[i].language);
+        cc_utf8 = utf8_init(ti.sub_charsets[i].charset);
         break;
       }
 
     if (!sub_charset_found) {
-      if (mm_io->get_byte_order() != BO_NONE)
+      if (io->get_byte_order() != BO_NONE)
         cc_utf8 = utf8_init("UTF-8");
       else
-        cc_utf8 = utf8_init(ti->sub_charset);
+        cc_utf8 = utf8_init(ti.sub_charset);
     }
 
-    ti->id = 0;                 // ID for this track.
-    global = mm_io->getline();  // [Script Info]
-    while (!mm_io->eof()) {
-      old_pos = mm_io->getFilePointer();
-      line = mm_io->getline();
+    ti.id = 0;                 // ID for this track.
+    global = io->getline();  // [Script Info]
+    while (!io->eof()) {
+      old_pos = io->getFilePointer();
+      line = io->getline();
 
       if (!strncasecmp(line.c_str(), "Dialogue: ", strlen("Dialogue: "))) {
         // End of global data - restore file position to just before this
         // line end bail out.
-        mm_io->setFilePointer(old_pos);
+        io->setFilePointer(old_pos);
         break;
       }
 
@@ -124,35 +103,37 @@ ssa_reader_c::ssa_reader_c(track_info_c *nti)
         strip(format);
       }
 
-      // Now just append the current line and some DOS style newlines.
-      // But not if we've already encountered the [Events] section.
-      if (section != 'e') {
-        global += "\r\n";
-        global += line;
-      }
+      global += "\r\n";
+      global += line;
     }
 
     if (format.size() == 0)
       throw error_c("ssa_reader: Invalid format. Could not find the "
                     "\"Format\" line in the \"[Events]\" section.");
 
-    textsubs_packetizer = new textsubs_packetizer_c(this, is_ass ? 
-                                                    MKV_S_TEXTASS :
-                                                    MKV_S_TEXTSSA,
-                                                    global.c_str(),
-                                                    global.length(), false,
-                                                    false, ti);
-  } catch (exception &ex) {
+  } catch (...) {
     throw error_c("ssa_reader: Could not open the source file.");
   }
   if (verbose)
-    mxinfo("Using SSA/ASS subtitle reader for %s.\n+-> Using "
-           "text subtitle output module for subtitles.\n", ti->fname);
+    mxinfo(FMT_FN "Using the SSA/ASS subtitle reader.\n", ti.fname.c_str());
+  parse_file();
 }
 
 ssa_reader_c::~ssa_reader_c() {
-  if (textsubs_packetizer != NULL)
-    delete textsubs_packetizer;
+  delete io;
+}
+
+void
+ssa_reader_c::create_packetizer(int64_t) {
+  if (NPTZR() != 0)
+    return;
+
+  add_packetizer(new textsubs_packetizer_c(this, is_ass ?  MKV_S_TEXTASS :
+                                           MKV_S_TEXTSSA, global.c_str(),
+                                           global.length(), false, false,
+                                           ti));
+  mxinfo(FMT_TID "Using the text subtitle output module.\n", ti.fname.c_str(),
+         (int64_t)0);
 }
 
 string
@@ -178,7 +159,7 @@ ssa_reader_c::parse_time(string &stime) {
     return -1;
 
   s = stime.substr(0, pos);
-  if (!parse_int(s.c_str(), th))
+  if (!parse_int(s, th))
     return -1;
   stime.erase(0, pos + 1);
 
@@ -187,7 +168,7 @@ ssa_reader_c::parse_time(string &stime) {
     return -1;
 
   s = stime.substr(0, pos);
-  if (!parse_int(s.c_str(), tm))
+  if (!parse_int(s, tm))
     return -1;
   stime.erase(0, pos + 1);
 
@@ -196,11 +177,11 @@ ssa_reader_c::parse_time(string &stime) {
     return -1;
 
   s = stime.substr(0, pos);
-  if (!parse_int(s.c_str(), ts))
+  if (!parse_int(s, ts))
     return -1;
   stime.erase(0, pos + 1);
 
-  if (!parse_int(stime.c_str(), tds))
+  if (!parse_int(stime, tds))
     return -1;
 
   return (tds * 10 + ts * 1000 + tm * 60 * 1000 + th * 60 * 60 * 1000) *
@@ -209,31 +190,21 @@ ssa_reader_c::parse_time(string &stime) {
 
 string
 ssa_reader_c::recode_text(vector<string> &fields) {
-  char *s;
-  string res;
-
   // TODO: Handle \fe encoding changes.
-  res = get_element("Text", fields);
-  s = to_utf8(cc_utf8, res.c_str());
-  res = s;
-  safefree(s);
-
-  return res;
+  return to_utf8(cc_utf8, get_element("Text", fields));
 }
 
-int
-ssa_reader_c::read(generic_packetizer_c *) {
+void
+ssa_reader_c::parse_file() {
   string line, stime, orig_line, comma;
-  int i, num;
+  int num;
   int64_t start, end;
-  vector<ssa_line_c> clines;
   vector<string> fields;
-  ssa_line_c cline;
 
-  num = 1;
+  num = 0;
 
   do {
-    line = mm_io->getline();
+    line = io->getline();
     orig_line = line;
     if (strncasecmp(line.c_str(), "Dialogue: ", strlen("Dialogue: ")))
       continue;
@@ -242,6 +213,8 @@ ssa_reader_c::read(generic_packetizer_c *) {
 
     // Split the line into fields.
     fields = split(line.c_str(), ",", format.size());
+    while (fields.size() < format.size())
+      fields.push_back(string(""));
 
     // Parse the start time.
     stime = get_element("Start", fields);
@@ -267,64 +240,45 @@ ssa_reader_c::read(generic_packetizer_c *) {
     // ReadOrder, Layer, Style, Name, MarginL, MarginR, MarginV, Effect, Text
 
     comma = ",";
-    line = comma + get_element("Layer", fields) + comma +
+    line = to_string(num) + comma + get_element("Layer", fields) + comma +
       get_element("Style", fields) + comma +
-      get_element("Name", fields) + comma + 
+      get_element("Name", fields) + comma +
       get_element("MarginL", fields) + comma +
       get_element("MarginR", fields) + comma +
       get_element("MarginV", fields) + comma +
       get_element("Effect", fields) + comma +
       recode_text(fields);
 
-    cline.line = safestrdup(line.c_str());
-    cline.start = start;
-    cline.end = end;
-    cline.num = num;
+    subs.add(start, end, line);
     num++;
+  } while (!io->eof());
 
-    clines.push_back(cline);
-  } while (!mm_io->eof());
+  subs.sort();
+}
 
-  stable_sort(clines.begin(), clines.end());
+file_status_e
+ssa_reader_c::read(generic_packetizer_c *,
+                   bool) {
+  if (subs.empty())
+    return FILE_STATUS_DONE;
 
-  for (i = 0; i < clines.size(); i++) {
-    char buffer[20];
-    // Let the packetizer handle this line.
-    mxprints(buffer, "%d", clines[i].num);
-    line = string(buffer) + string(clines[i].line);
-    textsubs_packetizer->process((unsigned char *)line.c_str(), 0,
-                                 clines[i].start,
-                                 clines[i].end - clines[i].start);
-    safefree(clines[i].line);
-  }
+  subs.process((textsubs_packetizer_c *)PTZR0);
 
-  textsubs_packetizer->flush();
-
-  return 0;
+  return subs.empty() ? FILE_STATUS_DONE : FILE_STATUS_MOREDATA;
 }
 
 int
-ssa_reader_c::display_priority() {
-  return DISPLAYPRIORITY_LOW;
-}
+ssa_reader_c::get_progress() {
+  int num_entries;
 
-static char wchar[] = "-\\|/-\\|/-";
-
-void
-ssa_reader_c::display_progress(bool) {
-  mxinfo("working... %c\r", wchar[act_wchar]);
-  act_wchar++;
-  if (act_wchar == strlen(wchar))
-    act_wchar = 0;
-}
-
-void
-ssa_reader_c::set_headers() {
-  textsubs_packetizer->set_headers();
+  num_entries = subs.get_num_entries();
+  if (num_entries == 0)
+    return 100;
+  return 100 * subs.get_num_processed() / num_entries;
 }
 
 void
 ssa_reader_c::identify() {
   mxinfo("File '%s': container: SSA/ASS\nTrack ID 0: subtitles "
-         "(SSA/ASS)\n", ti->fname);
+         "(SSA/ASS)\n", ti.fname.c_str());
 }
