@@ -1,22 +1,16 @@
 /*
-  mkvmerge GUI -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge GUI -- utility for splicing together matroska files
+   from component media subtypes
 
-  tab_settings.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
-  Parts of this code were written by Florian Wager <root@sirelvis.de>
+   $Id: tab_settings.h 2795 2005-02-19 15:42:31Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   declarations for the settings tab
 
-/*!
-    \file
-    \version $Id: tab_settings.h 1525 2004-03-09 16:35:22Z mosu $
-    \brief declarations for the settings tab
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #ifndef __TAB_SETTINGS_H
@@ -26,6 +20,10 @@
 #define ID_B_BROWSEMKVMERGE                15001
 #define ID_COB_PRIORITY                    15002
 #define ID_CB_AUTOSET_OUTPUT_FILENAME      15003
+#define ID_CB_ASK_BEFORE_OVERWRITING       15004
+#define ID_CB_ON_TOP                       15005
+#define ID_CB_NEW_AFTER_ADD_TO_JOBQUEUE    15006
+#define ID_CB_WARN_USAGE                   15007
 
 class tab_settings: public wxPanel {
   DECLARE_CLASS(tab_settings);
@@ -33,6 +31,9 @@ class tab_settings: public wxPanel {
 public:
   wxTextCtrl *tc_mkvmerge;
   wxCheckBox *cb_show_commandline, *cb_autoset_output_filename;
+  wxCheckBox *cb_ask_before_overwriting, *cb_on_top;
+  wxCheckBox *cb_filenew_after_add_to_jobqueue;
+  wxCheckBox *cb_warn_usage;
   wxComboBox *cob_priority;
 
 public:
@@ -41,6 +42,7 @@ public:
 
   void on_browse(wxCommandEvent &evt);
   void on_xyz_selected(wxCommandEvent &evt);
+  void on_on_top_selected(wxCommandEvent &evt);
 
   void load_preferences();
   void save_preferences();
