@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_ac3.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_ac3.cpp 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   AC3 output module
 
-/*!
-    \file
-    \version $Id: p_ac3.cpp 1449 2004-01-30 20:40:38Z mosu $
-    \brief AC3 output module
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include <stdlib.h>
@@ -27,28 +22,23 @@
 #include "ac3_common.h"
 #include "p_ac3.h"
 #include "matroska.h"
-#include "mkvmerge.h"
 
 using namespace libmatroska;
 
-ac3_packetizer_c::ac3_packetizer_c(generic_reader_c *nreader,
-                                   unsigned long nsamples_per_sec,
-                                   int nchannels,
-                                   int nbsid,
-                                   track_info_c *nti)
+ac3_packetizer_c::ac3_packetizer_c(generic_reader_c *_reader,
+                                   int _samples_per_sec,
+                                   int _channels,
+                                   int _bsid,
+                                   track_info_c &_ti)
   throw (error_c):
-  generic_packetizer_c(nreader, nti) {
-  packetno = 0;
-  bytes_output = 0;
-  samples_per_sec = nsamples_per_sec;
-  channels = nchannels;
-  bsid = nbsid;
+  generic_packetizer_c(_reader, _ti),
+  bytes_output(0), packetno(0),
+  samples_per_sec(_samples_per_sec), channels(_channels), bsid(_bsid) {
 
   set_track_type(track_audio);
-  if (use_durations)
-    set_track_default_duration((int64_t)(1536000000000.0 *ti->async.linear /
-                                            samples_per_sec));
-  duplicate_data_on_add(false);
+  set_track_default_duration((int64_t)(1536000000000.0 *ti.async.linear /
+                                       samples_per_sec));
+  enable_avi_audio_sync(true);
 }
 
 ac3_packetizer_c::~ac3_packetizer_c() {
@@ -78,6 +68,33 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
   if ((pos + ac3header->bytes) > size)
     return NULL;
 
+  if (pos > 0) {
+    bool warning_printed;
+
+    warning_printed = false;
+    if (packetno == 0) {
+      int64_t offset;
+
+      offset = handle_avi_audio_sync(pos, false);
+      if (offset != -1) {
+        mxinfo("The AC3 track %lld from '%s' contained %d bytes of non-AC3 "
+               "data at the beginning. This corresponds to a delay of %lldms. "
+               "This delay will be used instead of the non-AC3 data.\n",
+               ti.id, ti.fname.c_str(), pos, offset / 1000000);
+        warning_printed = true;
+      }
+    }
+    if (!warning_printed)
+      mxwarn("The AC3 track %lld from '%s' contained %d bytes of non-AC3 data "
+             "at the beginning which were skipped. The audio/video "
+             "synchronization may have been lost.\n", ti.id,
+             ti.fname.c_str(), pos);
+    byte_buffer.remove(pos);
+    packet_buffer = byte_buffer.get_buffer();
+    size = byte_buffer.get_size();
+    pos = 0;
+  }
+
   pins = 1536000000000.0 / samples_per_sec;
 
   if (needs_negative_displacement(pins)) {
@@ -86,16 +103,12 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
      * appropriate number of packets at the beginning.
      */
     displace(-pins);
-    byte_buffer.remove(pos + ac3header->bytes);
+    byte_buffer.remove(ac3header->bytes);
 
-    return NULL;
+    return get_ac3_packet(header, ac3header);
   }
 
-  if (verbose && (pos > 1))
-    mxwarn("ac3_packetizer: skipping %d bytes (no valid AC3 header "
-           "found). This might make audio/video go out of sync, but this "
-           "stream is damaged.\n", pos);
-  buf = (unsigned char *)safememdup(packet_buffer + pos, ac3header->bytes);
+  buf = (unsigned char *)safememdup(packet_buffer, ac3header->bytes);
 
   if (needs_positive_displacement(pins)) {
     /*
@@ -109,7 +122,7 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
     return buf;
   }
 
-  byte_buffer.remove(pos + ac3header->bytes);
+  byte_buffer.remove(ac3header->bytes);
 
   return buf;
 }
@@ -130,8 +143,7 @@ ac3_packetizer_c::set_headers() {
 }
 
 int
-ac3_packetizer_c::process(unsigned char *buf,
-                          int size,
+ac3_packetizer_c::process(memory_c &mem,
                           int64_t timecode,
                           int64_t,
                           int64_t,
@@ -143,23 +155,24 @@ ac3_packetizer_c::process(unsigned char *buf,
 
   debug_enter("ac3_packetizer_c::process");
 
-  add_to_buffer(buf, size);
+  add_to_buffer(mem.data, mem.size);
   while ((packet = get_ac3_packet(&header, &ac3header)) != NULL) {
     if (timecode == -1)
       my_timecode = (int64_t)(1000000000.0 * packetno * 1536 /
                               samples_per_sec);
     else
-      my_timecode = timecode + initial_displacement;
-    my_timecode = (int64_t)(my_timecode * ti->async.linear);
-    add_packet(packet, ac3header.bytes, my_timecode,
-               (int64_t)(1000000000.0 * 1536 * ti->async.linear /
+      my_timecode = timecode + ti.async.displacement;
+    my_timecode = (int64_t)(my_timecode * ti.async.linear);
+    memory_c mem(packet, ac3header.bytes, true);
+    add_packet(mem, my_timecode,
+               (int64_t)(1000000000.0 * 1536 * ti.async.linear /
                          samples_per_sec));
     packetno++;
   }
 
   debug_leave("ac3_packetizer_c::process");
 
-  return EMOREDATA;
+  return FILE_STATUS_MOREDATA;
 }
 
 void
@@ -168,13 +181,26 @@ ac3_packetizer_c::dump_debug_info() {
           packet_queue.size(), byte_buffer.get_size());
 }
 
+connection_result_e
+ac3_packetizer_c::can_connect_to(generic_packetizer_c *src) {
+  ac3_packetizer_c *asrc;
+
+  asrc = dynamic_cast<ac3_packetizer_c *>(src);
+  if (asrc == NULL)
+    return CAN_CONNECT_NO_FORMAT;
+  if ((samples_per_sec != asrc->samples_per_sec) ||
+      (channels != asrc->channels) || (bsid != asrc->bsid))
+    return CAN_CONNECT_NO_PARAMETERS;
+  return CAN_CONNECT_YES;
+}
+
 ac3_bs_packetizer_c::ac3_bs_packetizer_c(generic_reader_c *nreader,
                                          unsigned long nsamples_per_sec,
                                          int nchannels,
                                          int nbsid,
-                                         track_info_c *nti)
+                                         track_info_c &_ti)
   throw (error_c):
-  ac3_packetizer_c(nreader, nsamples_per_sec, nchannels, nbsid, nti) {
+  ac3_packetizer_c(nreader, nsamples_per_sec, nchannels, nbsid, _ti) {
   bsb_present = false;
 }
 
