@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_ac3.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_ac3.h 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   class definition for the AC3 output module
 
-/*!
-    \file
-    \version $Id: p_ac3.h 1303 2003-12-04 14:36:24Z mosu $
-    \brief class definition for the AC3 output module
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #ifndef __P_AC3_H
@@ -31,21 +26,25 @@
 class ac3_packetizer_c: public generic_packetizer_c {
 protected:
   int64_t bytes_output, packetno;
-  unsigned long samples_per_sec;
-  int channels, bsid;
+  int samples_per_sec, channels, bsid;
   byte_buffer_c byte_buffer;
 
 public:
-  ac3_packetizer_c(generic_reader_c *nreader, unsigned long nsamples_per_sec,
-                   int nchannels, int nbsid, track_info_c *nti)
+  ac3_packetizer_c(generic_reader_c *_reader, int _samples_per_sec,
+                   int _channels, int _bsid, track_info_c &_ti)
     throw (error_c);
   virtual ~ac3_packetizer_c();
 
-  virtual int process(unsigned char *buf, int size, int64_t timecode = -1,
+  virtual int process(memory_c &mem, int64_t timecode = -1,
                       int64_t length = -1, int64_t bref = -1,
                       int64_t fref = -1);
   virtual void set_headers();
 
+  virtual const char *get_format_name() {
+    return "AC3";
+  }
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src);
+
 protected:
   virtual unsigned char *get_ac3_packet(unsigned long *header,
                                         ac3_header_t *ac3header);
@@ -62,7 +61,7 @@ protected:
 public:
   ac3_bs_packetizer_c(generic_reader_c *nreader,
                       unsigned long nsamples_per_sec,
-                      int nchannels, int nbsid, track_info_c *nti)
+                      int nchannels, int nbsid, track_info_c &_ti)
     throw (error_c);
 
 protected:
