@@ -1,21 +1,17 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_dts.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_dts.cpp 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   DTS output module
 
-/*!
-    \file
-    \version $Id: p_dts.cpp 1434 2004-01-22 22:36:05Z mosu $
-    \brief DTS output module
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Peter Niemayer <niemayer@isg.de>.
+   Modified by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include <stdlib.h>
@@ -25,84 +21,23 @@
 
 #include "pr_generic.h"
 #include "dts_common.h"
-#include "p_dts.h"
 #include "matroska.h"
+#include "output_control.h"
+#include "p_dts.h"
 
 using namespace libmatroska;
 
-bool
-operator!=(const dts_header_t &l,
-           const dts_header_t &r) {
-  //if (l.frametype != r.frametype) return true;
-  //if (l.deficit_sample_count != r.deficit_sample_count) return true;
-  if (l.crc_present != r.crc_present)
-    return true;
-  if (l.num_pcm_sample_blocks != r.num_pcm_sample_blocks)
-    return true;
-  if (l.frame_byte_size != r.frame_byte_size)
-    return true;
-  if (l.audio_channels != r.audio_channels)
-    return true;
-  if (l.core_sampling_frequency != r.core_sampling_frequency)
-    return true;
-  if (l.transmission_bitrate != r.transmission_bitrate)
-    return true;
-  if (l.embedded_down_mix != r.embedded_down_mix)
-    return true;
-  if (l.embedded_dynamic_range != r.embedded_dynamic_range)
-    return true;
-  if (l.embedded_time_stamp != r.embedded_time_stamp)
-    return true;
-  if (l.auxiliary_data != r.auxiliary_data)
-    return true;
-  if (l.hdcd_master != r.hdcd_master)
-    return true;
-  if (l.extension_audio_descriptor != r.extension_audio_descriptor)
-    return true;
-  if (l.extended_coding != r.extended_coding)
-    return true;
-  if (l.audio_sync_word_in_sub_sub != r.audio_sync_word_in_sub_sub)
-    return true;
-  if (l.lfe_type != r.lfe_type)
-    return true;
-  if (l.predictor_history_flag != r.predictor_history_flag)
-    return true;
-  if (l.multirate_interpolator != r.multirate_interpolator)
-    return true;
-  if (l.encoder_software_revision != r.encoder_software_revision)
-    return true;
-  if (l.copy_history != r.copy_history)
-    return true;
-  if (l.source_pcm_resolution != r.source_pcm_resolution)
-    return true;
-  if (l.source_surround_in_es != r.source_surround_in_es)
-    return true;
-  if (l.front_sum_difference != r.front_sum_difference)
-    return true;
-  if (l.surround_sum_difference != r.surround_sum_difference)
-    return true;
-  if (l.dialog_normalization_gain != r.dialog_normalization_gain)
-    return true;
-  return false;
-}
-
-dts_packetizer_c::dts_packetizer_c(generic_reader_c *nreader,
+dts_packetizer_c::dts_packetizer_c(generic_reader_c *_reader,
                                    const dts_header_t &dtsheader,
-                                   track_info_c *nti)
-  throw (error_c): generic_packetizer_c(nreader, nti) {
-  //packetno = 0;
-  samples_written = 0;
-  bytes_written = 0;
-
-  packet_buffer = NULL;
-  buffer_size = 0;
-  skipping_is_normal = false;
-
-  first_header = dtsheader;
-  last_header = dtsheader;
+                                   track_info_c &_ti,
+                                   bool _get_first_header_later)
+  throw (error_c):
+  generic_packetizer_c(_reader, _ti),
+  samples_written(0), bytes_written(0), packet_buffer(NULL), buffer_size(0),
+  get_first_header_later(_get_first_header_later),
+  first_header(dtsheader), last_header(dtsheader), skipping_is_normal(false) {
 
   set_track_type(track_audio);
-  duplicate_data_on_add(false);
 }
 
 dts_packetizer_c::~dts_packetizer_c() {
@@ -167,6 +102,14 @@ dts_packetizer_c::get_dts_packet(dts_header_t &dtsheader) {
   if ((pos + dtsheader.frame_byte_size) > buffer_size)
     return 0;
 
+  if (get_first_header_later) {
+    first_header = dtsheader;
+    last_header = dtsheader;
+    get_first_header_later = false;
+    set_headers();
+    rerender_track_headers();
+  }
+
   if (dtsheader != last_header) {
     mxinfo("DTS header information changed! - New format:\n");
     print_dts_header(&dtsheader);
@@ -184,7 +127,7 @@ dts_packetizer_c::get_dts_packet(dts_header_t &dtsheader) {
     displace(-pins);
     remove_dts_packet(pos, dtsheader.frame_byte_size);
 
-    return 0;
+    return get_dts_packet(dtsheader);
   }
 
   if (verbose && (pos > 0) && !skipping_is_normal)
@@ -216,14 +159,17 @@ void
 dts_packetizer_c::set_headers() {
   set_codec_id(MKV_A_DTS);
   set_audio_sampling_freq((float)first_header.core_sampling_frequency);
-  set_audio_channels(first_header.audio_channels);
+  if ((first_header.lfe_type == dts_header_t::LFE_64) ||
+      (first_header.lfe_type == dts_header_t::LFE_128))
+    set_audio_channels(first_header.audio_channels + 1);
+  else
+    set_audio_channels(first_header.audio_channels);
 
   generic_packetizer_c::set_headers();
 }
 
 int
-dts_packetizer_c::process(unsigned char *buf,
-                          int size,
+dts_packetizer_c::process(memory_c &mem,
                           int64_t timecode,
                           int64_t,
                           int64_t,
@@ -236,7 +182,7 @@ dts_packetizer_c::process(unsigned char *buf,
   if (timecode != -1)
     my_timecode = timecode;
 
-  add_to_buffer(buf, size);
+  add_to_buffer(mem.data, mem.size);
 
   dts_header_t dtsheader;
   unsigned char *packet;
@@ -248,10 +194,11 @@ dts_packetizer_c::process(unsigned char *buf,
       my_timecode = (int64_t)(((double)samples_written * 1000000000.0) /
                               ((double)dtsheader.core_sampling_frequency));
     else
-      my_timecode = timecode + initial_displacement;
-    my_timecode = (int64_t)(my_timecode * ti->async.linear);
-    add_packet(packet, dtsheader.frame_byte_size, my_timecode,
-               (int64_t)(packet_len_in_ns * ti->async.linear));
+      my_timecode = timecode + ti.async.displacement;
+    my_timecode = (int64_t)(my_timecode * ti.async.linear);
+    memory_c mem(packet, dtsheader.frame_byte_size, true);
+    add_packet(mem, my_timecode, (int64_t)(packet_len_in_ns *
+                                           ti.async.linear));
 
     bytes_written += dtsheader.frame_byte_size;
     samples_written += get_dts_packet_length_in_core_samples(&dtsheader);
@@ -259,10 +206,21 @@ dts_packetizer_c::process(unsigned char *buf,
 
   debug_leave("dts_packetizer_c::process");
 
-  return EMOREDATA;
+  return FILE_STATUS_MOREDATA;
 }
 
 void
 dts_packetizer_c::dump_debug_info() {
   mxdebug("dts_packetizer_c: queue: %d\n", packet_queue.size());
 }
+
+connection_result_e
+dts_packetizer_c::can_connect_to(generic_packetizer_c *src) {
+  dts_packetizer_c *dsrc;
+
+  dsrc = dynamic_cast<dts_packetizer_c *>(src);
+  if (dsrc == NULL)
+    return CAN_CONNECT_NO_FORMAT;
+  // Hmm...
+  return CAN_CONNECT_YES;
+}
