@@ -1,21 +1,17 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_dts.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_dts.h 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   class definition for the DTS output module
 
-/*!
-    \file
-    \version $Id: p_dts.h 1303 2003-12-04 14:36:24Z mosu $
-    \brief class definition for the DTS output module
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Peter Niemayer <niemayer@isg.de>.
+   Modified by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #ifndef __P_DTS_H
@@ -31,24 +27,29 @@ class dts_packetizer_c: public generic_packetizer_c {
 private:
   int64_t samples_written, bytes_written;
 
+  unsigned char *packet_buffer;
   int buffer_size;
 
-  unsigned char *packet_buffer;
-
+  bool get_first_header_later;
   dts_header_t first_header;
   dts_header_t last_header;
 
 public:
   bool skipping_is_normal;
 
-  dts_packetizer_c(generic_reader_c *nreader, const dts_header_t &dts_header,
-                   track_info_c *nti) throw (error_c);
+  dts_packetizer_c(generic_reader_c *_reader, const dts_header_t &dts_header,
+                   track_info_c &_ti, bool _get_first_header_later = false)
+    throw (error_c);
   virtual ~dts_packetizer_c();
 
-  virtual int process(unsigned char *buf, int size, int64_t timecode = -1,
+  virtual int process(memory_c &mem, int64_t timecode = -1,
                       int64_t length = -1, int64_t bref = -1,
                       int64_t fref = -1);
   virtual void set_headers();
+  virtual const char *get_format_name() {
+    return "DTS";
+  }
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src);
 
 private:
   virtual void add_to_buffer(unsigned char *buf, int size);
