@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_passthrough.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_passthrough.h 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   class definition for the pass through output module
 
-/*!
-    \file
-    \version $Id: p_passthrough.h 1539 2004-03-13 12:16:49Z mosu $
-    \brief class definition for the pass through output module
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #ifndef __P_PASSTHROUGH_H
@@ -32,19 +27,24 @@ private:
   bool sync_to_keyframe, sync_complete_group;
 
 public:
-  passthrough_packetizer_c(generic_reader_c *nreader, track_info_c *nti)
+  passthrough_packetizer_c(generic_reader_c *_reader, track_info_c &_ti)
     throw (error_c);
 
-  virtual int process(unsigned char *buf, int size, int64_t timecode = -1,
+  virtual int process(memory_c &mem, int64_t timecode = -1,
                       int64_t duration = -1, int64_t bref = -1,
                       int64_t fref = -1);
-  virtual int process(unsigned char *buf, int size, int64_t timecode,
+  virtual int process(memory_c &mem, int64_t timecode,
                       int64_t duration, int64_t bref, int64_t fref,
                       bool duration_mandatory);
   virtual void set_headers();
   virtual void always_sync_complete_group(bool sync);
 
   virtual void dump_debug_info();
+
+  virtual const char *get_format_name() {
+    return "passthrough";
+  }
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src);
 };
 
 #endif // __P_PASSTHROUGH_H
