@@ -1,29 +1,24 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_vorbis.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_vorbis.cpp 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
+   Vorbis packetizer
+
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-/*!
-    \file
-    \version $Id: p_vorbis.cpp 1434 2004-01-22 22:36:05Z mosu $
-    \brief Vorbis packetizer
-    \author Moritz Bunkus <moritz@bunkus.org>
-*/
-
-#include "config.h"
+#include "os.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(COMP_MSC)
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
@@ -31,28 +26,27 @@
 #include <vorbis/codec.h>
 
 #include "common.h"
-#include "pr_generic.h"
-#include "p_vorbis.h"
 #include "matroska.h"
-#include "mkvmerge.h"
+#include "output_control.h"
+#include "p_vorbis.h"
 
 using namespace libmatroska;
 
-vorbis_packetizer_c::vorbis_packetizer_c(generic_reader_c *nreader,
+vorbis_packetizer_c::vorbis_packetizer_c(generic_reader_c *_reader,
                                          unsigned char *d_header,
                                          int l_header,
                                          unsigned char *d_comments,
                                          int l_comments,
                                          unsigned char *d_codecsetup,
                                          int l_codecsetup,
-                                         track_info_c *nti)
+                                         track_info_c &_ti)
   throw (error_c):
-  generic_packetizer_c(nreader, nti) {
+  generic_packetizer_c(_reader, _ti),
+  last_bs(0), samples(0), last_samples_sum(0), last_timecode(0),
+  timecode_offset(0) {
+
   int i;
 
-  packetno = 0;
-  last_bs = 0;
-  samples = 0;
   memset(headers, 0, 3 * sizeof(ogg_packet));
   headers[0].packet = (unsigned char *)safememdup(d_header, l_header);
   headers[1].packet = (unsigned char *)safememdup(d_comments, l_comments);
@@ -73,8 +67,8 @@ vorbis_packetizer_c::vorbis_packetizer_c(generic_reader_c *nreader,
   set_track_type(track_audio);
   if (use_durations)
     set_track_default_duration((int64_t)(1024000000000.0 *
-                                            ti->async.linear / vi.rate));
-  ti->async.displacement = initial_displacement;
+                                            ti.async.linear / vi.rate));
+  ti.async.displacement = initial_displacement;
 }
 
 vorbis_packetizer_c::~vorbis_packetizer_c() {
@@ -83,6 +77,8 @@ vorbis_packetizer_c::~vorbis_packetizer_c() {
   for (i = 0; i < 3; i++)
     if (headers[i].packet != NULL)
       safefree(headers[i].packet);
+  vorbis_info_clear(&vi);
+  vorbis_comment_clear(&vc);
 }
 
 void
@@ -132,28 +128,32 @@ vorbis_packetizer_c::set_headers() {
 }
 
 /*
- * Some notes - processing is straight-forward if no AV synchronization
- * is needed - the packet is simply stored in the Matroska file.
- * Unfortunately things are not that easy if AV sync is done. For a
- * negative displacement packets are simply discarded if their timecode
- * is set before the displacement. For positive displacements the packetizer
- * has to generate silence packets and put them into the Matroska file first.
- */
+   Some notes - processing is straight-forward if no AV synchronization
+   is needed - the packet is simply stored in the Matroska file.
+   Unfortunately things are not that easy if AV sync is done. For a
+   negative displacement packets are simply discarded if their timecode
+   is set before the displacement. For positive displacements the packetizer
+   has to generate silence packets and put them into the Matroska file first.
+*/
 int
-vorbis_packetizer_c::process(unsigned char *data,
-                             int size,
+vorbis_packetizer_c::process(memory_c &mem,
                              int64_t timecode,
                              int64_t,
                              int64_t,
                              int64_t) {
   unsigned char zero[2];
   ogg_packet op;
-  int64_t this_bs, samples_here, samples_needed;
+  int64_t this_bs, samples_here, samples_needed, expected_timecode, duration;
+  int64_t chosen_timecode;
 
   debug_enter("vorbis_packetizer_c::process");
 
+  // Remember the very first timecode we received.
+  if ((samples == 0) && (timecode > 0))
+    timecode_offset = timecode;
+
   // Positive displacement, first packet? Well then lets create silence.
-  if ((packetno == 0) && (initial_displacement > 0)) {
+  if ((samples == 0) && (initial_displacement > 0)) {
     // Create a fake packet so we can use vorbis_packet_blocksize().
     zero[0] = 0;
     zero[1] = 0;
@@ -170,49 +170,86 @@ vorbis_packetizer_c::process(unsigned char *data,
       samples += samples_here;
       last_bs = this_bs;
       samples_here = (this_bs + last_bs) / 4;
-      add_packet(zero, 2, samples * 1000 / vi.rate,
+      memory_c mem(zero, 2, false);
+      add_packet(mem, samples * 1000000000 / vi.rate,
                  samples_here * 1000000000 / vi.rate);
     }
+    last_samples_sum = samples;
   }
 
-  // Recalculate the timecode if needed.
-  if (timecode == -1) {
-    if (initial_displacement > 0)
-      timecode = samples * 1000000000 / vi.rate;
-    else
-      timecode = samples * 1000000000 / vi.rate + initial_displacement;
-  } else
-    timecode += initial_displacement;
-
-  // Handle the linear sync - simply multiply with the given factor.
-  timecode = (int64_t)((double)timecode * ti->async.linear);
-
   // Update the number of samples we have processed so that we can
   // calculate the timecode on the next call.
-  op.packet = data;
-  op.bytes = size;
+  op.packet = mem.data;
+  op.bytes = mem.size;
   this_bs = vorbis_packet_blocksize(&vi, &op);
   samples_here = (this_bs + last_bs) / 4;
-  samples += samples_here;
   last_bs = this_bs;
+  samples += samples_here;
 
-  // If a negative sync value was used we may have to skip this packet.
-  if (timecode < 0) {
-    debug_leave("vorbis_packetizer_c::process");
-    return EMOREDATA;
+  expected_timecode = last_timecode + last_samples_sum * 1000000000 / vi.rate +
+    timecode_offset;
+  timecode += initial_displacement;
+  if (initial_displacement < 0)
+    expected_timecode += initial_displacement;
+
+  if (timecode > (expected_timecode + 100000000)) {
+    chosen_timecode = timecode;
+    duration = timecode - last_timecode;
+    last_timecode = timecode;
+    last_samples_sum = 0;
+  } else {
+    chosen_timecode = expected_timecode;
+    duration = (int64_t)(samples_here * 1000000000 * ti.async.linear /
+                         vi.rate);
   }
 
-  mxverb(2, "Vorbis: samples_here: %lld\n", samples_here);
-  add_packet(data, size, (int64_t)timecode,
-             (int64_t)(samples_here * 1000000000 * ti->async.linear /
-                       vi.rate));
+  last_samples_sum += samples_here;
+
+  // Handle the linear sync - simply multiply with the given factor.
+  chosen_timecode = (int64_t)((double)chosen_timecode * ti.async.linear);
+
+  // If a negative sync value was used we may have to skip this packet.
+  if (chosen_timecode < 0) {
+    debug_leave("vorbis_packetizer_c::process");
+    return FILE_STATUS_MOREDATA;
+  }
+
+  mxverb(2, "Vorbis: samples_here at %lld (orig %lld expected %lld): %lld "
+         "(last_samples_sum: %lld)\n",
+         chosen_timecode, timecode, expected_timecode,
+         samples_here, last_samples_sum);
+  add_packet(mem, expected_timecode, duration);
 
   debug_leave("vorbis_packetizer_c::process");
 
-  return EMOREDATA;
+  return FILE_STATUS_MOREDATA;
 }
 
 void
 vorbis_packetizer_c::dump_debug_info() {
   mxdebug("vorbis_packetizer_c: queue: %d\n", packet_queue.size());
 }
+
+connection_result_e
+vorbis_packetizer_c::can_connect_to(generic_packetizer_c *src) {
+  vorbis_packetizer_c *vsrc;
+
+  vsrc = dynamic_cast<vorbis_packetizer_c *>(src);
+  if (vsrc == NULL)
+    return CAN_CONNECT_NO_FORMAT;
+  if ((vi.rate != vsrc->vi.rate) || (vi.channels != vsrc->vi.channels) ||
+      (headers[0].bytes != vsrc->headers[0].bytes) ||
+      (headers[1].bytes != vsrc->headers[1].bytes) ||
+      (headers[2].bytes != vsrc->headers[2].bytes) ||
+      (headers[0].packet == NULL) ||
+      (headers[1].packet == NULL) ||
+      (headers[2].packet == NULL) ||
+      (vsrc->headers[0].packet == NULL) ||
+      (vsrc->headers[1].packet == NULL) ||
+      (vsrc->headers[2].packet == NULL) ||
+      memcmp(headers[0].packet, vsrc->headers[0].packet, headers[0].bytes) ||
+      memcmp(headers[1].packet, vsrc->headers[1].packet, headers[1].bytes) ||
+      memcmp(headers[2].packet, vsrc->headers[2].packet, headers[2].bytes))
+    return CAN_CONNECT_NO_PARAMETERS;
+  return CAN_CONNECT_YES;
+}
