@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  p_vorbis.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: p_vorbis.h 2809 2005-02-24 21:05:35Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   class definition for the Vorbis packetizer
 
-/*!
-    \file
-    \version $Id: p_vorbis.h 1303 2003-12-04 14:36:24Z mosu $
-    \brief class definition for the Vorbis packetizer
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include "config.h"
@@ -34,25 +29,30 @@
 
 class vorbis_packetizer_c: public generic_packetizer_c {
 private:
-  int64_t last_bs, samples;
-  int packetno;
+  int64_t last_bs, samples, last_samples_sum, last_timecode, timecode_offset;
   vorbis_info vi;
   vorbis_comment vc;
   ogg_packet headers[3];
+
 public:
-  vorbis_packetizer_c(generic_reader_c *nreader,
+  vorbis_packetizer_c(generic_reader_c *_reader,
                       unsigned char *d_header, int l_header,
                       unsigned char *d_comments, int l_comments,
                       unsigned char *d_codecsetup, int l_codecsetup,
-                      track_info_c *nti) throw (error_c);
+                      track_info_c &_ti) throw (error_c);
   virtual ~vorbis_packetizer_c();
 
-  virtual int process(unsigned char *data, int size, int64_t timecode = -1,
+  virtual int process(memory_c &mem, int64_t timecode = -1,
                       int64_t length = -1, int64_t bref = -1,
                       int64_t fref = -1);
   virtual void set_headers();
 
   virtual void dump_debug_info();
+
+  virtual const char *get_format_name() {
+    return "Vorbis";
+  }
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src);
 };
 
 #endif  // __P_VORBIS_H
